//
// Joint Bilateral Upsampling
// Copyright (c) 2008
// University of Utah
// All rights reserved.
//
// This software is licensed under the BSD open-source license. See
// http://www.opensource.org/licenses/bsd-license.php for more detail.
//
// *************************************************************
// Redistribution and use in source and binary forms, with or 
// without modification, are permitted provided that the following 
// conditions are met:
//
// Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
//
// Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
//
// Neither the name of the University of Utah nor the names of 
// the contributors may be used to endorse or promote products derived 
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
// OF SUCH DAMAGE.

// final scaling value
uniform vec4 params;

// base image to be convolved
uniform sampler2D framebuffer;

uniform sampler2D depth;

void main()
{
  float curr_depth = texture2D(depth, gl_FragCoord.xy * params.xy).x;  
  float i,j;
  vec4 sum = vec4(0.0, 0.0, 0.0, 0.0);
  float weight = 0.0;
  float distance = 0.0;
  float domain_gaussian = 0.0;
  float range_gaussian = 0.0;
  float radius = params.z;
  float sigma = 2.0*radius*radius;
  float sample_depth = 0.0;
  float sigma2 = 2.0*params.w*params.w;
  vec2 pos;

  // Lookup surrounding fragments and add up values
  for (i = -radius; i <= radius; i+=1.0)
  {
    for (j = -radius; j <= radius; j+=1.0)
    {  
      pos.x = (gl_FragCoord.x+i)*params.x;
      pos.y = (gl_FragCoord.y+j)*params.y;
      vec4 sample = texture2D(framebuffer, pos);

      // domain
      distance = i*i+j*j;
      domain_gaussian = exp(-distance/sigma);

      // range
      sample_depth = texture2D(depth, pos).x;
      distance = (curr_depth-sample_depth)*(curr_depth-sample_depth);
      range_gaussian = exp(-distance/sigma2);

      sum += sample*domain_gaussian*range_gaussian; 
      weight += domain_gaussian*range_gaussian;
    }
  }
  
  gl_FragColor = sum/weight;
}
